@@ -35,13 +35,13 @@ setMethod(
   "show_implementation", signature(object = "system_basic"),
   function(object) {
     callNextMethod(object)
-    cat(sprintf(
-      "  %-18s: %s\n", "Short Side Rule", paste0(
-        quantity_variable(object@demand), " = min(",
-        prefixed_quantity_variable(object@demand), ", ",
-        prefixed_quantity_variable(object@supply), ")"
-      )
-    ))
+    cat(
+      labels = sprintf("  %-18s:", "Short Side Rule"),
+      quantity_variable(object@demand), " = min(",
+      prefixed_quantity_variable(object@demand), ", ",
+      prefixed_quantity_variable(object@supply), ")",
+      sep = "", fill = TRUE
+    )
   }
 )
 
